{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## FP-Growth\n",
    "\n",
    "The class `treeNode` is a class definition for the nodes of **FP-Tree** and has variables to hold the count and the name of the node. The `nodeLink` is used to link similar items and `parent` is used to link to the parent `treeNode` in the FP-Tree."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 1,
   "metadata": {
    "collapsed": true
   },
   "outputs": [],
   "source": [
    "class treeNode:\n",
    "    def __init__(self, nameValue, numOccur, parentNode):\n",
    "        self.name = nameValue\n",
    "        self.count = numOccur\n",
    "        self.nodeLink = None\n",
    "        self.parent = parentNode      #needs to be updated\n",
    "        self.children = {} \n",
    "    \n",
    "    def inc(self, numOccur):\n",
    "        self.count += numOccur\n",
    "        \n",
    "    def disp(self, ind=1):\n",
    "        print '  '*ind, self.name, ' ', self.count\n",
    "        for child in self.children.values():\n",
    "            child.disp(ind+1)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "The `createTree` function takes a *dataset* and the *minimum support* value and builds an *FP-tree*, includuing the tree itself and a header table. "
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "metadata": {
    "collapsed": true
   },
   "outputs": [],
   "source": [
    "def createTree(dataSet, minSup=1): #create FP-tree from dataset but don't mine\n",
    "    headerTable = {}\n",
    "    #go over dataSet twice\n",
    "    \n",
    "    #first pass counts frequency of occurance\n",
    "    for trans in dataSet:\n",
    "        for item in trans:\n",
    "            headerTable[item] = headerTable.get(item, 0) + dataSet[trans]\n",
    "        \n",
    "    #remove items not meeting minSup\n",
    "    for k in headerTable.keys():  \n",
    "        if headerTable[k] < minSup: \n",
    "            del(headerTable[k])\n",
    "            \n",
    "    freqItemSet = set(headerTable.keys())\n",
    "    #print 'freqItemSet: ',freqItemSet\n",
    "    \n",
    "    #if no items meet min support -->get out\n",
    "    if len(freqItemSet) == 0: \n",
    "        return None, None  \n",
    "    \n",
    "    #reformat headerTable to use Node link \n",
    "    for k in headerTable:\n",
    "        headerTable[k] = [headerTable[k], None] \n",
    "    #print 'headerTable: ',headerTable\n",
    "    \n",
    "    retTree = treeNode('Null Set', 1, None) #create tree\n",
    "    \n",
    "    #go through dataset 2nd time\n",
    "    for tranSet, count in dataSet.items():  \n",
    "        localD = {}\n",
    "        \n",
    "        #put transaction items in order\n",
    "        for item in tranSet:  \n",
    "            if item in freqItemSet:\n",
    "                localD[item] = headerTable[item][0]\n",
    "                \n",
    "        if len(localD) > 0:\n",
    "            orderedItems = [v[0] for v in sorted(localD.items(), key=lambda p: p[1], reverse=True)]\n",
    "            #populate tree with ordered freq itemset\n",
    "            updateTree(orderedItems, retTree, headerTable, count)\n",
    "    \n",
    "    #return tree and header table\n",
    "    return retTree, headerTable "
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "1"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 3,
   "metadata": {
    "collapsed": true
   },
   "outputs": [],
   "source": [
    "def updateTree(items, inTree, headerTable, count):\n",
    "    #check if orderedItems[0] in retTree.children\n",
    "    if items[0] in inTree.children:\n",
    "        #incrament count\n",
    "        inTree.children[items[0]].inc(count) \n",
    "    else:   \n",
    "        #add items[0] to inTree.children\n",
    "        inTree.children[items[0]] = treeNode(items[0], count, inTree)\n",
    "        #update header table \n",
    "        if headerTable[items[0]][1] == None:\n",
    "            headerTable[items[0]][1] = inTree.children[items[0]]\n",
    "        else:\n",
    "            updateHeader(headerTable[items[0]][1], inTree.children[items[0]])\n",
    "    \n",
    "    #call updateTree() with remaining ordered items\n",
    "    if len(items) > 1:\n",
    "        updateTree(items[1::], inTree.children[items[0]], headerTable, count)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 4,
   "metadata": {
    "collapsed": true
   },
   "outputs": [],
   "source": [
    "#this version does not use recursion\n",
    "#Do not use recursion to traverse a linked list!\n",
    "#the linked list might be too long and you reach the limits :-(\n",
    "def updateHeader(nodeToTest, targetNode):   \n",
    "    while (nodeToTest.nodeLink != None):    \n",
    "        nodeToTest = nodeToTest.nodeLink\n",
    "    nodeToTest.nodeLink = targetNode"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Let's test the code!"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 5,
   "metadata": {
    "collapsed": true
   },
   "outputs": [],
   "source": [
    "# a simple dataset to illustrate the algorithm\n",
    "def loadSimpDat():\n",
    "    simpDat = [['r', 'z', 'h', 'j', 'p'],\n",
    "               ['z', 'y', 'x', 'w', 'v', 'u', 't', 's'],\n",
    "               ['z'],\n",
    "               ['r', 'x', 'n', 'o', 's'],\n",
    "               ['y', 'r', 'x', 'z', 'q', 't', 'p'],\n",
    "               ['y', 'z', 'x', 'e', 'q', 's', 't', 'm']]\n",
    "    return simpDat"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 6,
   "metadata": {
    "collapsed": true
   },
   "outputs": [],
   "source": [
    "# We need the dataset in a special format, let's do it\n",
    "# a dictionary with transactions as key and count as value\n",
    "def createInitSet(dataSet):\n",
    "    retDict = {}\n",
    "    for trans in dataSet:\n",
    "        retDict[frozenset(trans)] = 1\n",
    "    return retDict"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Now we are ready to test it"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 7,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "[['r', 'z', 'h', 'j', 'p'],\n",
       " ['z', 'y', 'x', 'w', 'v', 'u', 't', 's'],\n",
       " ['z'],\n",
       " ['r', 'x', 'n', 'o', 's'],\n",
       " ['y', 'r', 'x', 'z', 'q', 't', 'p'],\n",
       " ['y', 'z', 'x', 'e', 'q', 's', 't', 'm']]"
      ]
     },
     "execution_count": 7,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "simpDat = loadSimpDat()\n",
    "simpDat"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 8,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "{frozenset({'e', 'm', 'q', 's', 't', 'x', 'y', 'z'}): 1,\n",
       " frozenset({'n', 'o', 'r', 's', 'x'}): 1,\n",
       " frozenset({'z'}): 1,\n",
       " frozenset({'s', 't', 'u', 'v', 'w', 'x', 'y', 'z'}): 1,\n",
       " frozenset({'p', 'q', 'r', 't', 'x', 'y', 'z'}): 1,\n",
       " frozenset({'h', 'j', 'p', 'r', 'z'}): 1}"
      ]
     },
     "execution_count": 8,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "initSet = createInitSet(simpDat)\n",
    "initSet"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 9,
   "metadata": {
    "collapsed": true
   },
   "outputs": [],
   "source": [
    "myFPtree, myHeaderTable = createTree(initSet, 3) # min-sup = 3"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 10,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "   Null Set   1\n",
      "     x   1\n",
      "       s   1\n",
      "         r   1\n",
      "     z   5\n",
      "       x   3\n",
      "         y   3\n",
      "           s   2\n",
      "             t   2\n",
      "           r   1\n",
      "             t   1\n",
      "       r   1\n"
     ]
    }
   ],
   "source": [
    "myFPtree.disp()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 11,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "{'r': [3, <__main__.treeNode instance at 0x0000000003CD33C8>],\n",
       " 's': [3, <__main__.treeNode instance at 0x0000000003CD3288>],\n",
       " 't': [3, <__main__.treeNode instance at 0x0000000003CD3308>],\n",
       " 'x': [4, <__main__.treeNode instance at 0x0000000003CD31C8>],\n",
       " 'y': [3, <__main__.treeNode instance at 0x0000000003CD32C8>],\n",
       " 'z': [5, <__main__.treeNode instance at 0x0000000003CBEFC8>]}"
      ]
     },
     "execution_count": 11,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "myHeaderTable"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Now, let's mine the FP-tree\n",
    "\n",
    "We need to do three basic steps:\n",
    "1. get conditional pattern bases from the FP-tree\n",
    "2. from the condditional pattern base, construct a conditional FP-tree\n",
    "3. recursively repeat steps 1 and 2 until the tree contains a single item\n",
    "\n",
    "Let's find the conditional pattern base, it is a collection of paths from the root that end with an specific item, i.e. it is a prefix path."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 12,
   "metadata": {
    "collapsed": true
   },
   "outputs": [],
   "source": [
    "#treeNode comes from header table\n",
    "def findPrefixPath(treeNode):\n",
    "    condPats = {}\n",
    "    while treeNode != None:\n",
    "        prefixPath = []\n",
    "        ascendTree(treeNode, prefixPath)\n",
    "        if len(prefixPath) > 1: \n",
    "            condPats[frozenset(prefixPath[1:])] = treeNode.count\n",
    "        treeNode = treeNode.nodeLink\n",
    "    return condPats"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 13,
   "metadata": {
    "collapsed": true
   },
   "outputs": [],
   "source": [
    "#ascends from leaf node to root\n",
    "def ascendTree(leafNode, prefixPath):\n",
    "    if leafNode.parent != None:\n",
    "        prefixPath.append(leafNode.name)\n",
    "        ascendTree(leafNode.parent, prefixPath)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Let's test it"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 14,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "{frozenset({'z'}): 3}"
      ]
     },
     "execution_count": 14,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "findPrefixPath(myHeaderTable['x'][1])"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 15,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "{}"
      ]
     },
     "execution_count": 15,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "findPrefixPath(myHeaderTable['z'][1])"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 16,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "{frozenset({'s', 'x'}): 1, frozenset({'z'}): 1, frozenset({'x', 'y', 'z'}): 1}"
      ]
     },
     "execution_count": 16,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "findPrefixPath(myHeaderTable['r'][1])"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Now, let's mine the tree"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 17,
   "metadata": {
    "collapsed": true
   },
   "outputs": [],
   "source": [
    "def mineTree(inTree, headerTable, minSup, preFix, freqItemList, verbose=False):\n",
    "    #(sort header table)\n",
    "    bigL = [v[0] for v in sorted(headerTable.items(), key=lambda p: p[1])]\n",
    "    #start from bottom of header table\n",
    "    for basePat in bigL:\n",
    "        newFreqSet = preFix.copy()\n",
    "        newFreqSet.add(basePat)\n",
    "        #print 'finalFrequent Item: ',newFreqSet    \n",
    "        \n",
    "        #append to set\n",
    "        freqItemList.append(newFreqSet)\n",
    "        condPattBases = findPrefixPath(headerTable[basePat][1]) # basePat\n",
    "        #print 'condPattBases :',basePat, condPattBases\n",
    "        \n",
    "        #2. construct cond FP-tree from cond. pattern base\n",
    "        myCondTree, myHead = createTree(condPattBases, minSup)\n",
    "        #print 'head from conditional tree: ', myHead\n",
    "        \n",
    "        #3. mine cond. FP-tree\n",
    "        if myHead != None: \n",
    "            if verbose:\n",
    "                print 'conditional tree for: ',newFreqSet\n",
    "                myCondTree.disp(1)            \n",
    "            mineTree(myCondTree, myHead, minSup, newFreqSet, freqItemList)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "OK! It's time to spark the code ..."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 19,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "conditional tree for:  set(['s'])\n",
      "   Null Set   1\n",
      "     x   3\n",
      "conditional tree for:  set(['y'])\n",
      "   Null Set   1\n",
      "     x   3\n",
      "       z   3\n",
      "conditional tree for:  set(['t'])\n",
      "   Null Set   1\n",
      "     y   3\n",
      "       x   3\n",
      "         z   3\n",
      "conditional tree for:  set(['x'])\n",
      "   Null Set   1\n",
      "     z   3\n"
     ]
    }
   ],
   "source": [
    "# list to store the frequent itsets\n",
    "freqItems = []\n",
    "\n",
    "mineTree(myFPtree, myHeaderTable, 3, set([]), freqItems, verbose=True)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 20,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "[{'s'},\n",
       " {'s', 'x'},\n",
       " {'y'},\n",
       " {'x', 'y'},\n",
       " {'y', 'z'},\n",
       " {'x', 'y', 'z'},\n",
       " {'t'},\n",
       " {'t', 'y'},\n",
       " {'t', 'x'},\n",
       " {'t', 'x', 'y'},\n",
       " {'t', 'z'},\n",
       " {'t', 'y', 'z'},\n",
       " {'t', 'x', 'z'},\n",
       " {'t', 'x', 'y', 'z'},\n",
       " {'r'},\n",
       " {'x'},\n",
       " {'x', 'z'},\n",
       " {'z'}]"
      ]
     },
     "execution_count": 20,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "freqItems"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Work in the Wild\n",
    "\n",
    "a helper to delete useless characters and URLS and generate a list of words"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 21,
   "metadata": {
    "collapsed": true
   },
   "outputs": [],
   "source": [
    "import re\n",
    "\n",
    "def textParse(bigString):\n",
    "    urlsRemoved = re.sub('(http(s)*:[/][/]|www.)([a-z]|[A-Z]|[0-9]|[/.]|[~])*', '', bigString)    \n",
    "    listOfTokens = re.split(r'\\W*', urlsRemoved)\n",
    "    return [tok.lower() for tok in listOfTokens if len(tok) > 2]"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 22,
   "metadata": {
    "collapsed": true
   },
   "outputs": [],
   "source": [
    "# reading the dataset\n",
    "tweets = []\n",
    "with open('tweets200000.txt', 'r') as f:\n",
    "    for line in f:\n",
    "        tweets.append(textParse(line))"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 23,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "['wind',\n",
       " 'mph',\n",
       " 'nne',\n",
       " 'barometer',\n",
       " 'rising',\n",
       " 'slowly',\n",
       " 'temperature',\n",
       " 'rain',\n",
       " 'today',\n",
       " 'humidity']"
      ]
     },
     "execution_count": 23,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "tweets[0]"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 24,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "225318"
      ]
     },
     "execution_count": 24,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "len(tweets)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 25,
   "metadata": {
    "collapsed": true
   },
   "outputs": [],
   "source": [
    "min_sup = 5\n",
    "\n",
    "initSetTweets = createInitSet(tweets[0:1500])\n",
    "tweetFPtree, tweetHeaderTable = createTree(initSetTweets, min_sup)\n",
    "\n",
    "tweetFreqList = []\n",
    "mineTree(tweetFPtree, tweetHeaderTable, min_sup, set([]), tweetFreqList)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 26,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "16483"
      ]
     },
     "execution_count": 26,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "len(tweetFreqList)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 27,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "0 27 8\n"
     ]
    }
   ],
   "source": [
    "import numpy as np\n",
    "\n",
    "length_of_tweets = [len(tw) for tw in tweets]\n",
    "print np.min(length_of_tweets), np.max(length_of_tweets), np.sum(length_of_tweets)/len(length_of_tweets)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 28,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "[10, 8, 0, 2, 9, 5, 7, 13, 4, 16]"
      ]
     },
     "execution_count": 28,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "length_of_tweets[:10]"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 29,
   "metadata": {
    "collapsed": false,
    "scrolled": true
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "[['wind',\n",
       "  'mph',\n",
       "  'nne',\n",
       "  'barometer',\n",
       "  'rising',\n",
       "  'slowly',\n",
       "  'temperature',\n",
       "  'rain',\n",
       "  'today',\n",
       "  'humidity'],\n",
       " ['pausa',\n",
       "  'pro',\n",
       "  'caf',\n",
       "  'antes',\n",
       "  'embarcar',\n",
       "  'ximo',\n",
       "  'trippolisontheroad',\n",
       "  'danipolisviaja'],\n",
       " [],\n",
       " ['pause', 'for'],\n",
       " ['good',\n",
       "  'morning',\n",
       "  'morning',\n",
       "  'saturday',\n",
       "  'diner',\n",
       "  'breakfast',\n",
       "  'nucorpsofcadetsring',\n",
       "  'ring',\n",
       "  'college'],\n",
       " ['gratefuldead', 'recordstoredayus', 'toms', 'music', 'trade'],\n",
       " ['egg', 'muffin', 'rocket', 'baby', 'bakery', 'rocketbabybaker', 'wauwatosa'],\n",
       " ['lyricwaters',\n",
       "  'should',\n",
       "  'gave',\n",
       "  'the',\n",
       "  'neighbor',\n",
       "  'buzz',\n",
       "  'got',\n",
       "  'ice',\n",
       "  'cream',\n",
       "  'and',\n",
       "  'moms',\n",
       "  'baked',\n",
       "  'goodies'],\n",
       " ['the', 'way', 'mamaroneck', 'mamaroneck'],\n",
       " ['hiring',\n",
       "  'read',\n",
       "  'about',\n",
       "  'our',\n",
       "  'latest',\n",
       "  'job',\n",
       "  'opening',\n",
       "  'here',\n",
       "  'retail',\n",
       "  'sales',\n",
       "  'consultant',\n",
       "  'cwa',\n",
       "  'mob',\n",
       "  'bryn',\n",
       "  'mawr',\n",
       "  'retail']]"
      ]
     },
     "execution_count": 29,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "tweets[:10]"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 31,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "['\"Wind 3.2 mph NNE. Barometer 30.20 in, Rising slowly. Temperature 49.3 \\xf8F. Rain today 0.00 in. Humidity 32%\"\\n', '\"Pausa pro caf\\x82 antes de embarcar no pr\\xa2ximo v\\x93o. #trippolisontheroad #danipolisviaja \\n', '\\n', 'Pause for? https://t.co/PhcJ4oYktP\"\\n', 'Good. Morning. #morning #Saturday #diner #VT #breakfast #nucorpsofcadetsring #ring #college? https://t.co/dBZ7dbwX6f\\n', '@gratefuldead recordstoredayus ?????? @ TOMS MUSIC TRADE https://t.co/CURRmn6iJo\\n', '\"Egg in a muffin!!! (@ Rocket Baby Bakery - @rocketbabybaker in Wauwatosa, WI) https://t.co/mwfhrcxtRp\"\\n', \"@lyricwaters should've gave the neighbor  a buzz. Iv got ice cream and moms baked goodies ??\\n\", '\"On the way to CT! (@ Mamaroneck, NY in Mamaroneck, NY) https://t.co/6rpe6MXDkB\"\\n', \"We're #hiring! Read about our latest #job opening here: Retail Sales Consultant [CWA MOB] Bryn Mawr PA - https://t.co/bBwxSPsL4f #Retail\\n\"]\n"
     ]
    }
   ],
   "source": [
    "with open('tweets200000.txt', 'r') as f:\n",
    "    print f.readlines()[:10]"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 33,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "[{'cincinnati'},\n",
       " {'trainee'},\n",
       " {'hiring', 'trainee'},\n",
       " {'job', 'trainee'},\n",
       " {'hiring', 'job', 'trainee'},\n",
       " {'retail', 'trainee'},\n",
       " {'hiring', 'retail', 'trainee'},\n",
       " {'job', 'retail', 'trainee'},\n",
       " {'hiring', 'job', 'retail', 'trainee'},\n",
       " {'jobs', 'trainee'},\n",
       " {'hiring', 'jobs', 'trainee'},\n",
       " {'job', 'jobs', 'trainee'},\n",
       " {'hiring', 'job', 'jobs', 'trainee'},\n",
       " {'jobs', 'retail', 'trainee'},\n",
       " {'hiring', 'jobs', 'retail', 'trainee'},\n",
       " {'job', 'jobs', 'retail', 'trainee'},\n",
       " {'hiring', 'job', 'jobs', 'retail', 'trainee'},\n",
       " {'fallschurch'},\n",
       " {'line'},\n",
       " {'cafe'}]"
      ]
     },
     "execution_count": 33,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "tweetFreqList[0:20]"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 34,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "[['r', 'z', 'h', 'j', 'p'],\n",
       " ['z', 'y', 'x', 'w', 'v', 'u', 't', 's'],\n",
       " ['z'],\n",
       " ['r', 'x', 'n', 'o', 's'],\n",
       " ['y', 'r', 'x', 'z', 'q', 't', 'p'],\n",
       " ['y', 'z', 'x', 'e', 'q', 's', 't', 'm']]"
      ]
     },
     "execution_count": 34,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "simpDat"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 35,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "5"
      ]
     },
     "execution_count": 35,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "min_sup"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 36,
   "metadata": {
    "collapsed": true
   },
   "outputs": [],
   "source": [
    "min_sup = 2"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 37,
   "metadata": {
    "collapsed": true
   },
   "outputs": [],
   "source": [
    "f1 = {}\n",
    "for rec in simpDat:\n",
    "    for item in rec:\n",
    "        if item not in f1: \n",
    "            f1[item] = 1\n",
    "        else:\n",
    "            f1[item] += 1"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 38,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "{'e': 1,\n",
       " 'h': 1,\n",
       " 'j': 1,\n",
       " 'm': 1,\n",
       " 'n': 1,\n",
       " 'o': 1,\n",
       " 'p': 2,\n",
       " 'q': 2,\n",
       " 'r': 3,\n",
       " 's': 3,\n",
       " 't': 3,\n",
       " 'u': 1,\n",
       " 'v': 1,\n",
       " 'w': 1,\n",
       " 'x': 4,\n",
       " 'y': 3,\n",
       " 'z': 5}"
      ]
     },
     "execution_count": 38,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "f1"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 42,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "{'p': 2, 'q': 2, 'r': 3, 's': 3, 't': 3, 'x': 4, 'y': 3, 'z': 5}"
      ]
     },
     "execution_count": 42,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "for item in f1.keys():\n",
    "    if f1[item] < min_sup:\n",
    "        del f1[item]\n",
    "        \n",
    "f1"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 43,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "[('q', 'p'),\n",
       " ('q', 's'),\n",
       " ('q', 'r'),\n",
       " ('q', 't'),\n",
       " ('q', 'y'),\n",
       " ('q', 'x'),\n",
       " ('q', 'z'),\n",
       " ('p', 'q'),\n",
       " ('p', 's'),\n",
       " ('p', 'r'),\n",
       " ('p', 't'),\n",
       " ('p', 'y'),\n",
       " ('p', 'x'),\n",
       " ('p', 'z'),\n",
       " ('s', 'q'),\n",
       " ('s', 'p'),\n",
       " ('s', 'r'),\n",
       " ('s', 't'),\n",
       " ('s', 'y'),\n",
       " ('s', 'x'),\n",
       " ('s', 'z'),\n",
       " ('r', 'q'),\n",
       " ('r', 'p'),\n",
       " ('r', 's'),\n",
       " ('r', 't'),\n",
       " ('r', 'y'),\n",
       " ('r', 'x'),\n",
       " ('r', 'z'),\n",
       " ('t', 'q'),\n",
       " ('t', 'p'),\n",
       " ('t', 's'),\n",
       " ('t', 'r'),\n",
       " ('t', 'y'),\n",
       " ('t', 'x'),\n",
       " ('t', 'z'),\n",
       " ('y', 'q'),\n",
       " ('y', 'p'),\n",
       " ('y', 's'),\n",
       " ('y', 'r'),\n",
       " ('y', 't'),\n",
       " ('y', 'x'),\n",
       " ('y', 'z'),\n",
       " ('x', 'q'),\n",
       " ('x', 'p'),\n",
       " ('x', 's'),\n",
       " ('x', 'r'),\n",
       " ('x', 't'),\n",
       " ('x', 'y'),\n",
       " ('x', 'z'),\n",
       " ('z', 'q'),\n",
       " ('z', 'p'),\n",
       " ('z', 's'),\n",
       " ('z', 'r'),\n",
       " ('z', 't'),\n",
       " ('z', 'y'),\n",
       " ('z', 'x')]"
      ]
     },
     "execution_count": 43,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "c2 = []\n",
    "for i in f1:\n",
    "    for j in f1:\n",
    "        if i != j:\n",
    "            c2.append(tuple([i,j]))\n",
    "            \n",
    "c2"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {
    "collapsed": true
   },
   "outputs": [],
   "source": []
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 2",
   "language": "python",
   "name": "python2"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 2
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython2",
   "version": "2.7.13"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 0
}
